%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Tutorial One  - Exercise One%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%% 1. Generate from a student distribution %%%%%%%%%%%

t_mu = -1;  % Expectation of the student
N_draws = 5000; % Number of draws
df = 10;  % Degree of freedom 
t_var = 2; % Variance of the student equal to 2

%%%%% Using the Matlab student command %%%%%%%%%%%

t_draws = t_mu + sqrt(t_var)*trnd(df,5000,1);
Empirical_mean = mean(t_draws);
Empirical_std = std(t_draws);
disp(['**** Empirical Mean of the student with an expectation of : ' num2str(t_mu)])
disp(Empirical_mean)
disp(['**** Empirical standard deviation of the student with a variance equal to : ' num2str(t_var)])
disp(Empirical_std)


%%%%% Using the relationship with the Gamma distribution (slide 143)%%%%%%%%%%%
I_gam_draws = 1./gamrnd(df/2,2/df,N_draws,1);
t_draws_bis = t_mu + sqrt(I_gam_draws).*normrnd(0,1,N_draws,1)*sqrt(t_var);

disp(['**** Empirical Mean of the student with an expectation of : ' num2str(t_mu)])
disp(mean(t_draws_bis))
disp(['**** Empirical standard deviation of the student with a variance equal to : ' num2str(t_var)])
disp(std(t_draws_bis))


%%%%% 2. Maximum likelihood estimate of the expectation of a student distribution %%%%%%%%%%%
option = optimset('MaxIter',1e7,'MaxFunEvals',1e7);
t_mu_0 = 8;
t_var_0 = 3;
df_0 = 20;
Init_point = [t_mu_0 t_var_0 df_0]';
[MLE,~,~,~,~,hessian]=fminunc(@(val_param) -1*log_like_stud(t_draws,val_param),Init_point);

disp('Maximum Likelihood Estimates of t_mu, t_var and t_df')
disp(MLE)

disp('Standard errors of t_mu, t_var and t_df')
std_MLE = sqrt(var(t_draws)/N_draws);

%%%%% 3. Moments of the student distribution given a diffuse prior and a Normal prior %%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Deterministic approach %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%
%%%% Choice of the hyper-parameters of the Normal prior
%%%%%%%
norm_mu_prior = 5;
norm_var_prior = 0.1;

%%%%%%%
%%%% Evaluation of the posterior kernel on a grid of points
%%%% Mapping from the real line to [0,1] in order to integrate
%%%%%%%
step = 0.0005;
max_value = 0.33;
min_value = 0.22;
tau = (min_value:step:max_value)';
N_int = max(size(tau));
mapping = -log((1-tau)./tau);
grad = 1./((1-tau).*tau);
grad2 = ones(N_int,1) + log(tau./(1-tau)) + 1./((1-tau));
diff_prior = zeros(N_int,1);
Normal_prior = zeros(N_int,1);
diff_prior_without_grad = zeros(N_int,1);
Normal_prior_without_grad = zeros(N_int,1);
for i=1:N_int
    param = [mapping(i);t_var;df];
    log_dens = log_like_stud(t_draws,param);
    diff_prior_without_grad(i) = log_dens;
    Normal_prior_without_grad(i) = log_dens -0.5*log(2*pi*norm_var_prior)-0.5*(mapping(i)-norm_mu_prior)^2/norm_var_prior;
    diff_prior(i,1) = log_dens + log(grad(i));
    Normal_prior(i,1) = log_dens -0.5*log(2*pi*norm_var_prior)-0.5*(mapping(i)-norm_mu_prior)^2/norm_var_prior + log(grad(i));
    
end
max_diff = max(diff_prior);
diff_prior = exp(diff_prior-max_diff);
diff_prior_without_grad = exp(diff_prior_without_grad-max_diff);
max_Normal = max(Normal_prior);
Normal_prior = exp(Normal_prior-max_Normal);
Normal_prior_without_grad =  exp(Normal_prior_without_grad-max_Normal);

%%%%%%%
%%%% Trapezoidal and Simpsons integrations
%%%%%%%
trapez = zeros(2,1);
simpsons = zeros(2,1);


trapez_weight = [1;ones(N_int-2,1)*2;1];
trapez(1) = (max_value-min_value)*(trapez_weight'*diff_prior)/(2*N_int);
trapez(2) = (max_value-min_value)*(trapez_weight'*Normal_prior)/(2*N_int);

A=[4 2];
simpsons_weight = [1, repmat(A, 1, floor(0.5*(N_int)-1)), 4, 1]';
simpsons(1) = step*(simpsons_weight'*diff_prior)/3;
simpsons(2) = step*(simpsons_weight'*Normal_prior)/3;

log_diff_trap_cst = log(trapez(1)) + max_diff;
log_Normal_trap_cst = log(trapez(2)) + max_Normal;
trap_pdf = zeros(N_int,2);
trap_pdf(:,1) = diff_prior_without_grad/trapez(1);
trap_pdf(:,2) = Normal_prior_without_grad/trapez(2);

plot(mapping,trap_pdf),set(gca,'Xlim',[min(mapping) max(mapping)]),title('Trapezoidal integration rule'); %% Plot of the two densities.


Simpsons_pdf = zeros(N_int,2);
Simpsons_pdf(:,1) = diff_prior_without_grad/simpsons(1);
Simpsons_pdf(:,2) = Normal_prior_without_grad/simpsons(2);

figure;
plot(mapping,Simpsons_pdf),set(gca,'Xlim',[min(mapping) max(mapping)]),title('Simpsons integration rule'); %% Plot of the two densities.


%%%%%%%%%%%%%%%%%%%%%
%%%%% Posterior expectation of mu under deterministic integrations
%%%%% Integration by Trapezoidal and Simpsons rules.
%%%%%%%%%%%%%%%%%%%%%

diff_Exp = (max_value-min_value)*(trapez_weight'*(trap_pdf(:,1).*mapping.*grad))/(2*N_int);
Normal_Exp = (max_value-min_value)*(trapez_weight'*(trap_pdf(:,2).*mapping.*grad))/(2*N_int);

Simp_diff_Exp = step*(simpsons_weight'*(Simpsons_pdf(:,1).*mapping.*grad))/(3);%step*(simpsons_weight'*(Simpsons_pdf(:,1).*tau.*grad))/3;
Simp_Normal_Exp = step*(simpsons_weight'*(Simpsons_pdf(:,2).*mapping.*grad))/(3);

disp(['*** Expectation obtained from trapezoidal and Simpsons rule for diffuse prior : ' num2str([diff_Exp Simp_diff_Exp])])
disp(['*** Expectation obtained from trapezoidal and Simpsons rule for Normal prior : ' num2str([Normal_Exp Simp_Normal_Exp])])
%%%%%%%%%%%%%%%%%%%%%
%%%%% Posterior variance of mu under deterministic integrations
%%%%%%%%%%%%%%%%%%%%%

diff_Var = (max_value-min_value)*(trapez_weight'*(trap_pdf(:,1).*grad.*(mapping-diff_Exp).^2))/(2*N_int);
Normal_Var = (max_value-min_value)*(trapez_weight'*(trap_pdf(:,2).*grad.*(mapping-Normal_Exp).^2))/(2*N_int);

Simp_diff_Var = step*(simpsons_weight'*(Simpsons_pdf(:,1).*((mapping-Simp_diff_Exp).^2).*grad))/(3);%step*(simpsons_weight'*(Simpsons_pdf(:,1).*tau.*grad))/3;
Simp_Normal_Var = step*(simpsons_weight'*(Simpsons_pdf(:,2).*((mapping-Simp_Normal_Exp).^2).*grad))/(3);

disp(['*** Posterior Standard Deviations of Mu obtained from trapezoidal rule for diffuse prior : ' num2str([sqrt(diff_Var) sqrt(Simp_diff_Var)])])
disp(['*** Posterior Standard Deviations of Mu obtained from trapezoidal rule for Normal prior : ' num2str([sqrt(Normal_Var) sqrt(Simp_Normal_Var)])])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Importance Sampling %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Choice of the proposal distribution : Normal distribution with 3*std_MLE as standard deviation
N_IS = 15000;
prop_var = 9*std_MLE(1)^2;
prop_draw = MLE(1) + randn(N_IS,1)*sqrt(prop_var); %% Draws from the proposal distribution (i.e. N(0,9 sigma_{MLE}^2)
diff_prior_IS = zeros(N_IS,1);
Normal_prior_IS =zeros(N_IS,1);
prop_dens_IS = zeros(N_IS,1);
log_dens_IS = zeros(N_IS,1);
MLE_mu = MLE(1);
for i=1:N_IS
    progressbar(i/N_IS);
    param = [prop_draw(i);t_var;df];
    log_dens_IS(i) = log_like_stud(t_draws,param); %% Evaluation of the likelihood.
    prop_dens_IS(i) = -0.5*log(2*pi*prop_var) - 0.5*(MLE_mu-prop_draw(i))^2/prop_var; %% Evaluation of the proposal.
    diff_prior_IS(i) = log_dens_IS(i); %% Not informative prior.
    Normal_prior_IS(i) = log_dens_IS(i) -0.5*log(2*pi*norm_var_prior)-0.5*(prop_draw(i)-norm_mu_prior)^2/norm_var_prior; %% Evaluation of the Normal prior.
end
%%%%%%%%%%%%%%%
%%% Computation of the Normalizing constant under a NIP
%%%%%%%%%%%%%%%
diff_max = max(diff_prior_IS-prop_dens_IS);
diff_weight = exp(diff_prior_IS-prop_dens_IS-diff_max);
log_diff_cst = log(mean(diff_weight)) + diff_max; %%%% Normalizing constant
diff_weight = diff_weight/sum(diff_weight); %%% Weight for each particle
diff_ESS = 1/(diff_weight'*diff_weight); %%% Effective sample size (belongs to [1 N_IS] : the higher, the better diversification of the particles.
Id_IS = zeros(N_IS,1);
for i=1:N_IS
    ind = multinomialrnd(diff_weight);
    Id_IS(i) = prop_draw(ind);
end
%%%%%%%%%%%%%%%
%%% Computation of the Normalizing constant under a Normal prior
%%%%%%%%%%%%%%%
Normal_max = max(Normal_prior_IS-prop_dens_IS);
Normal_weight = exp(Normal_prior_IS-prop_dens_IS-Normal_max); 
log_Normal_cst = log(mean(Normal_weight)) + Normal_max; %%%% Normalizing constant
Normal_weight = Normal_weight/sum(Normal_weight); %%% Weight for each particle
Normal_ESS = 1/(Normal_weight'*Normal_weight); %%% Effective sample size (belongs to [1 N_IS] : the higher, the better diversification of the particles.

diff_Exp_IS = prop_draw'*diff_weight;
Normal_Exp_IS = prop_draw'*Normal_weight;
disp(['*** Expectations obtained from IS for diffuse and Normal priors : ' num2str([diff_Exp_IS Normal_Exp_IS])])
diff_Var_IS = ((prop_draw-diff_Exp_IS).^2)'*diff_weight;
Normal_Var_IS = ((prop_draw-Normal_Exp_IS).^2)'*Normal_weight;
disp(['*** Posterior Standard Deviations obtained from IS for diffuse and Normal priors : ' num2str([sqrt(diff_Var_IS) sqrt(Normal_Var_IS)])])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Metropolis-Hasting %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nb_MCMC = N_IS; %% Number of MCMC iterations
mu_MCMC = zeros(nb_MCMC,2); 
mu_MCMC(1,:) = [prop_draw(1) prop_draw(1)]; %% Initial points of the MCMC.
log_dens_MCMC = zeros(nb_MCMC,2);
log_dens_MCMC(1,:) = [log_dens_IS(1) log_dens_IS(1)]; %% Initial kernel densities.
prop_dens_MCMC(1,:) = [prop_dens_IS(1) prop_dens_IS(1)]; %% Initial proposal densities.
Normal_prior_MCMC = zeros(nb_MCMC,1);
Normal_prior_MCMC(1) = -0.5*log(2*pi*norm_var_prior)-0.5*(prop_draw(1)-norm_mu_prior)^2/norm_var_prior; %% Initial prior density under a Norma prior.
accept_rate = zeros(2,1);
for i=2:nb_MCMC
    progressbar(i/nb_MCMC);    
    %%%%%%%%%%%
    %%%%%% Acceptance ratio under a NIP
    %%%%%%%%%%%
    if(exp(log_dens_IS(i)-log_dens_MCMC(i-1,1)+prop_dens_MCMC(i-1,1)-prop_dens_IS(i))>rand())
        mu_MCMC(i,1) = prop_draw(i);
        log_dens_MCMC(i,1) = log_dens_IS(i);
        prop_dens_MCMC(i,1) = prop_dens_IS(i);
        accept_rate(1) = accept_rate(1) +1;
    else %% Rejection of the draw
        mu_MCMC(i,1) = mu_MCMC(i-1,1);
        log_dens_MCMC(i,1) = log_dens_MCMC(i-1,1);
        prop_dens_MCMC(i,1) = prop_dens_MCMC(i-1,1);
    end
    %%%%%%%%%%%
    %%%%%% Acceptance ratio under a Normal prior with expectation :
    %%%%%% norm_mu_prior and variance norm_var_prior
    %%%%%%%%%%%
    Normal_prior_MCMC(i) = -0.5*log(2*pi*norm_var_prior)-0.5*(prop_draw(i)-norm_mu_prior)^2/norm_var_prior;
    if(exp(log_dens_IS(i)+Normal_prior_MCMC(i)-log_dens_MCMC(i-1,2)-Normal_prior_MCMC(i-1)+prop_dens_MCMC(i-1,2)-prop_dens_IS(i))>rand())
        mu_MCMC(i,2) = prop_draw(i);
        log_dens_MCMC(i,2) = log_dens_IS(i);
        prop_dens_MCMC(i,2) = prop_dens_IS(i);
        accept_rate(2) = accept_rate(2) +1;
    else %% Rejection of the draw
        mu_MCMC(i,2) = mu_MCMC(i-1,2);
        log_dens_MCMC(i,2) = log_dens_MCMC(i-1,2);
        prop_dens_MCMC(i,2) = prop_dens_MCMC(i-1,2);
        Normal_prior_MCMC(i) = Normal_prior_MCMC(i-1);
    end
end
accept_rate = accept_rate/(nb_MCMC-1);
disp(['*** Posterior Expectations obtained from MCMC for diffuse and Normal priors : ' num2str(mean(mu_MCMC))])
disp(['*** Posterior Standard Deviations obtained from MCMC for diffuse and Normal priors : ' num2str(std(mu_MCMC))])
%%%%%%%%%%%%%%%%%
%%% Autocorrelation functions of the two posterior samples
%%%%%%%%%%%%%%%%%
figure;
subplot(2,1,1),autocorr(mu_MCMC(:,1),100),title('Sample ACF of posterior draws under a diffuse prior');
subplot(2,1,2),autocorr(mu_MCMC(:,2),100),title('Sample ACF of posterior draws under a Normal prior');
%%%%%%%%%%%%%%%%%
%%% Cusum plots of the two posterior samples
%%%%%%%%%%%%%%%%%
figure;
[struct_cusum] = cusum_plot(mu_MCMC,1);
figure;
%%%%%%%%%%%%%%%%%
%%% Kernel densities of the two different prior distributions
%%%%%%%%%%%%%%%%%
[f] = ksdensity(mu_MCMC(:,1),mapping);
[f2] = ksdensity(mu_MCMC(:,2),mapping);
plot(mapping,f),set(gca,'Xlim',[min(mapping) max(mapping)]),title('Kernel density of the MCMC posterior distribution of mu under a diffuse prior');
hold on,plot(mapping,f2,'g'),set(gca,'Xlim',[min(mapping) max(mapping)]),title('Kernel density of the MCMC posterior distribution of mu under a diffuse prior');
