function [Freq] = Tut1_Ex2(N_draw)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Solution of Exercise 2 from Tutorial 1.
%%%% For a given number of draws, Bayes and Frequentist inferences are
%%%% carried out.
%%%% Outputs : 
%%%% Two structures : Bayes and Freq
%%%% They contain the mean, covariance matrixes, probabilities of being in
%%%% A_1 or A and the ratio mean and median for different numbers of draws.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Choice of the Normal expectation and Normal variance-covariance
%%%%% matrix %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mu = [1 ; -1];
Sigma = [0.2 0.25;0.25 0.64];
%%%%%%%%%%%
%%% Draws from a Multivariate Normal distribution
%%%%%%%%%%%
post_draw = mvnrnd(mu,Sigma,N_draw);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inference %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
diff_N = 100:500:N_draw;
dim_diff_N = max(size(diff_N));
Freq.true_prob_A1 = zeros(dim_diff_N,1);
Freq.true_prob_A = zeros(dim_diff_N,1);
Freq.prob_A1 = zeros(dim_diff_N,1);
Freq.prob_A = zeros(dim_diff_N,1);
Freq.quotient = zeros(dim_diff_N,2);
Freq.Expectation = zeros(dim_diff_N,2);
Freq.Cov_mat = zeros(2,2,dim_diff_N);
for i=1:dim_diff_N
    Freq.Expectation(i,:) = mean(post_draw(1:diff_N(i),:));
    Freq.Cov_mat(:,:,i) = cov(post_draw(1:diff_N(i),:));
    Freq.prob_A1(i) = sum((post_draw(1:diff_N(i),1)>0.5).*(post_draw(1:diff_N(i),1)<1.2))/diff_N(i);
    Freq.prob_A(i) = sum((post_draw(1:diff_N(i),1)>0.5).*(post_draw(1:diff_N(i),1)<1.2).*(post_draw(1:diff_N(i),2)>-1.5).*(post_draw(1:diff_N(i),2)<0))/diff_N(i);
    Freq.quotient(i,1) = mean(post_draw(1:diff_N(i),1)./post_draw(1:diff_N(i),2));
    Freq.quotient(i,2) = median(post_draw(1:diff_N(i),1)./post_draw(1:diff_N(i),2));
    Freq.true_prob_A1(i) = normcdf(1.2,mu(1),sqrt(Sigma(1,1)))-normcdf(0.5,mu(1),sqrt(Sigma(1,1)));
    Freq.true_prob_A(i) = 0.3571;%mvncdf([0.5 -1.5],[1.2 0],mu',Sigma);
end
disp('true and estimated prob_A1')
disp(Freq.true_prob_A1(1));
disp(Freq.prob_A1(dim_diff_N));

disp('true and estimated prob_A')
disp(Freq.true_prob_A(1));
disp(Freq.prob_A(dim_diff_N));

Freq.diff_N = diff_N;
plot(Freq.diff_N,Freq.true_prob_A1,'linewidth',5),hold on,plot(Freq.diff_N,Freq.prob_A1,'r','linewidth',5),legend('Prob of belonging to A_1','Estimated Prob of belonging to A_1'),set(gca,'Ylim',[0 1]);
set(gca,'fontSize',14)
figure;
plot(Freq.diff_N,Freq.true_prob_A,'linewidth',5),hold on,plot(Freq.diff_N,Freq.prob_A,'-*r','linewidth',5),legend('Prob of belonging to A','Estimated Prob of belonging to A'),set(gca,'Ylim',[0 1]);
set(gca,'fontSize',14)
figure;
plot(Freq.diff_N,Freq.quotient,'linewidth',5),legend('Posterior quotient mean \theta_1/\theta_2','Posterior quotient median \theta_1/\theta_2');
set(gca,'fontSize',14)

